#ifndef CORE_H
#define CORE_H

//    This file is part of JustAudio.

//    JustAudio is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustAudio is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with BashWire under the GPL.txt file. If not, see
//    <http://www.gnu.org/licenses/>.

#include <QAction>
#include <QObject>
#include <QMediaPlayer>
#include <QIODevice>
#include <QMessageBox>
#include <QAudioOutput>
#include <QStringList>
#include <QFileInfo>
#include <QList>
#include <QAction>
#include <QMediaPlayer>
#include <QFileSystemWatcher>
#include <QApplication>
#include <QCheckBox>
#include <QSlider>
#include <QLabel>
#include <QRect>
#include <QDir>

#include "aud_file.h"

#define SEGMENT_SIZE 10
#define PROCESS_FILE "process.txt"
#define CONFIG_FILE  "conf.csv"

class Core : public QObject
{
    Q_OBJECT

private:

    enum ConfValues
    {
        VOLUME,
        PLAY_DIR_CHECK,
        MUTE
    };

    QMediaPlayer *mplayer;
    QAction      *playAction;
    QAction      *pauseAction;
    QAction      *stopAction;
    QAction      *nextAction;
    QAction      *prevAction;
    QAction      *muteAction;
    QAction      *playDirAction;
    QSlider      *volumeSlider;
    QSlider      *seekSlider;
    QLabel       *currentFile;
    QLabel       *seekDisp;
    QStringList  *exts;
    bool          tracking;

    void updateSeekDisp();
    void fileDir(char direction);

private slots:

    void statusChanged(QMediaPlayer::MediaStatus state);
    void stateChanged(QMediaPlayer::PlaybackState state);
    void processfileChanged(const QString &path);
    void setDuration(int dur);
    void setPosition(int pos);
    void setVolume(int lvl);
    void nextFile();
    void prevFile();
    void volumeUp();
    void volumeDown();
    void seekerPress();
    void seekerRelease();

public:

    void setPlayer(QMediaPlayer *player);
    void setAudioOutput(QAudioOutput *audoOut);
    void setFileWatcher(QFileSystemWatcher *watcher);
    void setGuiActions(QAction *playAct, QAction *pauseAct, QAction *stopAct, QAction *nextAct, QAction *prevAct, QAction *muteAct, QAction *dir);
    void setGuiElements(QSlider *volSlider, QSlider *skSlider, QLabel *nameDisp, QLabel *seekLabel);
    void play(const QString &path);

    Core(QObject *parent, QStringList *filesup);
    ~Core();

signals:

    void start();
    void pause();
    void stop();
    void setSeek(qint64 pos);
    void setFile(const QUrl &url);
    void setVol(float value);
    void muteState(bool state);
};

#endif // CORE_H
