#include "gui.h"

//    This file is part of JustAudio.

//    JustAudio is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustAudio is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with BashWire under the GPL.txt file. If not, see
//    <http://www.gnu.org/licenses/>.

GUI::GUI(QApplication *app, QStringList *filesup, QWidget *parent): QMainWindow(parent)
{
    Q_UNUSED(app);

    exts = filesup;

    setup();
}

void GUI::openDialog()
{
    QFileDialog fileDialog;
    QString     filterString = tr("Audio Files (");

    for (int i = 0; i < exts->size(); ++i)
    {
        filterString.append(exts->value(i) + " ");
    }

    filterString = filterString.trimmed();

    filterString.append(")");

    fileDialog.setFileMode(QFileDialog::ExistingFile);
    fileDialog.setNameFilter(filterString);
    fileDialog.setDirectory(QDir::homePath());

    if (fileDialog.exec())
    {
        emit openFile(fileDialog.selectedFiles()[0]);
    }
}

void GUI::setup()
{
    auto mainWidget = new QWidget(this);
    auto mainLayout = new QVBoxLayout(mainWidget);

    auto seekWidget = new QWidget(this);
    auto seekLayout = new QHBoxLayout(seekWidget);

    auto contWidget = new QWidget(this);
    auto contLayout = new QHBoxLayout(contWidget);

    auto voluWidget = new QWidget(this);
    auto voluLayout = new QVBoxLayout(voluWidget);

    auto volWidActi = new QWidgetAction(this);
    auto volumeMenu = new QMenu(this);

    seeker    = new QSlider(this);
    volSlide  = new QSlider(this);
    seekLabel = new QLabel(this);
    fileDisp  = new QLabel(this);

    playBtn  = new SVGIcon("play", tr("Play"), true, this);
    pauseBtn = new SVGIcon("pause", tr("Pause"), true, this);
    stopBtn  = new SVGIcon("stop", tr("Stop"), false, this);
    prevBtn  = new SVGIcon("prev", tr("Previous file in dir"), false, this);
    nextBtn  = new SVGIcon("next", tr("Next file in dir"), false, this);
    openBtn  = new SVGIcon("open", tr("Open"), false, this);
    muteBtn  = new SVGIcon("mute", tr("Mute"), true, this);
    dirBtn   = new SVGIcon("playlist", tr("Play files in dir"), true, this);
    lvlsBtn  = new SVGIcon("levels", tr("Volume level"), false, this);

    auto fnt = seekLabel->font();

    fnt.setPointSize(5);

    fileDisp->setWordWrap(true);
    fileDisp->setFixedWidth(288);
    seeker->setOrientation(Qt::Horizontal);
    seeker->setFixedWidth(288);
    seekLabel->setFont(fnt);
    volSlide->setOrientation(Qt::Vertical);
    lvlsBtn->setMenu(volumeMenu);

    seekLayout->addWidget(seeker);
    seekLayout->addWidget(seekLabel);

    voluLayout->addWidget(new QLabel("+", this));
    voluLayout->addWidget(volSlide);
    voluLayout->addWidget(new QLabel("-", this));

    volWidActi->setDefaultWidget(voluWidget);
    volumeMenu->addAction(volWidActi);

    contLayout->addWidget(playBtn);
    contLayout->addWidget(pauseBtn);
    contLayout->addWidget(stopBtn);
    contLayout->addWidget(prevBtn);
    contLayout->addWidget(nextBtn);
    contLayout->addWidget(dirBtn);
    contLayout->addWidget(openBtn);
    contLayout->addWidget(muteBtn);
    contLayout->addWidget(lvlsBtn);

    mainLayout->addWidget(fileDisp, 0, Qt::AlignHCenter);
    mainLayout->addWidget(seekWidget, 0, Qt::AlignHCenter);
    mainLayout->addWidget(contWidget, 0, Qt::AlignHCenter);

    connect(openBtn->defaultAction(), &QAction::triggered, this, &GUI::openDialog);
    connect(dirBtn->defaultAction(), &QAction::triggered, nextBtn, &QToolButton::setEnabled);
    connect(dirBtn->defaultAction(), &QAction::triggered, prevBtn, &QToolButton::setEnabled);

    setCentralWidget(mainWidget);
}

QLabel  *GUI::seekDisp() {return seekLabel;}
QLabel  *GUI::fileName() {return fileDisp;}
QSlider *GUI::seek()     {return seeker;}
QSlider *GUI::volume()   {return volSlide;}
QAction *GUI::play()     {return playBtn->defaultAction();}
QAction *GUI::pause()    {return pauseBtn->defaultAction();}
QAction *GUI::stop()     {return stopBtn->defaultAction();}
QAction *GUI::prev()     {return prevBtn->defaultAction();}
QAction *GUI::next()     {return nextBtn->defaultAction();}
QAction *GUI::mute()     {return muteBtn->defaultAction();}
QAction *GUI::dir()      {return nextBtn->defaultAction();}

void GUI::showLicense()
{
    QDialog dialog;
    QFile   file(":/gpl", this);

    file.open(QFile::ReadOnly);

    QPlainTextEdit *text   = new QPlainTextEdit(file.readAll());
    QVBoxLayout    *layout = new QVBoxLayout(&dialog);
    QRect           rect   = QApplication::primaryScreen()->geometry();

    layout->addWidget(text);
    text->setReadOnly(true);
    text->setWordWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);

    file.close();
    dialog.setMinimumWidth(rect.width() / 4);
    dialog.setMinimumHeight(rect.height() - (rect.height() / 6));
    dialog.exec();
}

