#ifndef GUI_H
#define GUI_H

//    This file is part of JustAudio.

//    JustAudio is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustAudio is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with BashWire under the GPL.txt file. If not, see
//    <http://www.gnu.org/licenses/>.

#include <QMainWindow>
#include <QAction>
#include <QFileDialog>
#include <QCheckBox>
#include <QLabel>
#include <QSlider>
#include <QMenu>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QWidgetAction>
#include <QWidget>
#include <QToolButton>
#include <QPlainTextEdit>
#include <QRect>
#include <QApplication>
#include <QCursor>
#include <QIcon>
#include <QMouseEvent>
#include <QPixmap>

#include "svg_icon.h"

class GUI : public QMainWindow
{
    Q_OBJECT

private slots:

    void openDialog();
    void showLicense();

private:

    QStringList *exts;
    QLabel      *fileDisp;
    QLabel      *seekLabel;
    QSlider     *seeker;
    QSlider     *volSlide;
    QMenu       *volumeMenu;
    SVGIcon     *playBtn;
    SVGIcon     *pauseBtn;
    SVGIcon     *stopBtn;
    SVGIcon     *prevBtn;
    SVGIcon     *nextBtn;
    SVGIcon     *muteBtn;
    SVGIcon     *openBtn;
    SVGIcon     *dirBtn;
    SVGIcon     *lvlsBtn;

    void setup();

public:

    explicit GUI(QApplication *app, QStringList *filesup, QWidget *parent = nullptr);

    QAction *play();
    QAction *pause();
    QAction *stop();
    QAction *next();
    QAction *prev();
    QAction *mute();
    QAction *dir();
    QSlider *volume();
    QSlider *seek();
    QLabel  *fileName();
    QLabel  *seekDisp();

signals:

    void openFile(const QString &path);
};

#endif // GUI_H
