#include <QApplication>
#include <QMediaPlayer>
#include <QSystemTrayIcon>
#include <QObject>
#include <QFileSystemWatcher>
#include <QSharedMemory>
#include <QDir>
#include <QFile>
#include <QIcon>

#include "aud_file.h"
#include "core.h"
#include "gui.h"

//    This file is part of JustAudio.

//    JustAudio is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustAudio is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with BashWire under the GPL.txt file. If not, see
//    <http://www.gnu.org/licenses/>.

QStringList audExtensions()
{
    QStringList ret;

    // the media plugins are not checked for file format support. this is just an arbitrary list
    // of audio file extentions that are used when filtering for files to play. if a file fails
    // to play, the application will remain in silence or move on to the next file if nextCheck
    // is enabled.

    ret.append("*.mp3");
    ret.append("*.ogg");
    ret.append("*.flac");
    ret.append("*.wav");
    ret.append("*.au");
    ret.append("*.aif");
    ret.append("*.mid");
    ret.append("*.rmi");
    ret.append("*.m4a");
    ret.append("*.aiff");
    ret.append("*.ape");
    ret.append("*.wv");
    ret.append("*.3gp");
    ret.append("*.aa");
    ret.append("*.aac");
    ret.append("*.aax");
    ret.append("*.act");
    ret.append("*.amr");
    ret.append("*.au");
    ret.append("*.awb");
    ret.append("*.dct");
    ret.append("*.gsm");
    ret.append("*.m4b");
    ret.append("*.oga");
    ret.append("*.mogg");
    ret.append("*.ra");
    ret.append("*.rm");
    ret.append("*.raw");

    return ret;
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    app.setApplicationName("JustAudio");
    app.setApplicationVersion("1.0.0");

    QString path = QDir::homePath() + QDir::separator();

#ifndef Q_OS_WIN32

    path.append(QString(".config") + QDir::separator());

#else

    path.append(QString("AppData") + QDir::separator());
    path.append(QString("Local") + QDir::separator());

#endif

    QDir dir;

    path.append(QApplication::applicationName());
    dir.mkpath(path);

    QDir::setCurrent(path);

    if (!QFile::exists(PROCESS_FILE))
    {
        // QFileSystemWatcher will not work if the PROCESS_FILE file does not exists so
        // a base file need to be created to insure it connects to a file.

        QFile file(PROCESS_FILE, &app);

        file.open(QFile::WriteOnly | QFile::Truncate);
        file.write(" ");
        file.close();
    }

    // using a shared memory segment to determine if an instance of the app is running.
    // QSharedMemory::create() returns false if that is the case and true if not.
    // if an instance of the app is already runnning, the requested file path to playback
    // is written to PROCESS_FILE, that modification to the file is picked up by
    // QFileSystemWatcher will then read by the currently running instance and starts
    // playback of the file path written to PROCESS_FILE.

    QSharedMemory sharedMem(app.applicationName(), &app);

    if (true)//sharedMem.create(SEGMENT_SIZE))
    {
        auto exts = audExtensions();

        QFileSystemWatcher processFileWatcher(&app);
        GUI                gui(&app, &exts);
        QMediaPlayer       player(&app);
        QAudioOutput       audio(&app);
        Core               core(&app, &exts);

        player.setAudioOutput(&audio);
        core.setFileWatcher(&processFileWatcher);
        core.setPlayer(&player);
        core.setAudioOutput(&audio);
        core.setGuiActions(gui.play(), gui.pause(), gui.stop(), gui.next(), gui.prev(), gui.mute(), gui.dir());
        core.setGuiElements(gui.volume(), gui.seek(), gui.fileName(), gui.seekDisp());

        processFileWatcher.addPath(PROCESS_FILE);

        QObject::connect(&gui, &GUI::openFile, &core, &Core::play);

        gui.show();

        if (app.arguments().size() > 1)
        {
            core.play(app.arguments()[1]);
        }

        return app.exec();
    }
    else
    {
        if (app.arguments().size() > 1)
        {
            QFile file(PROCESS_FILE, &app);

            file.open(QFile::WriteOnly | QFile::Truncate);
            file.write(app.arguments()[1].toUtf8());
            file.close();
        }

        return 0;
    }
}
